package cn.gtmap.bdcdj.core.encrypt;

import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.lang3.StringUtils;
import org.dom4j.Document;
import org.dom4j.Element;
import org.dom4j.io.SAXReader;
import org.dom4j.tree.DefaultText;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.core.io.Resource;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

public class DbEncryptXmlConfig {
    private Resource configUrl;
    private static final Logger LOGGER = LoggerFactory.getLogger(DbEncryptXmlConfig.class);
    public Resource getConfigUrl() {
        return configUrl;
    }

    //全局加解密字段
    private List<String> globalEnableFields ;
    //map中全局加解密字段的例外（在例外中不加解密）
    private List<Map> globalDisableEntityFields;
    //entity中全局加解密字段的例外（在例外中不加解密）
    private List<Map> globalDisableMapFields;
    //sql加解密字段
    private List<Map> sqlFileds;
    //sql仅加密字段
    private List<Map> sqlEncryptFileds;
    //sql仅解密字段
    private List<Map> sqlDecryptFileds;
    //实体加解密字段
    private List<Map> entityFileds;
    //接口加解密列表
    private List<String> interfaceEncryptList;
    private String interfaceSm2DecodeKey;//接口加解密时，sm2解密的私钥
    private String interfaceSm2EncodeKey;//接口加解密时，sm2加密的公钥
    public String getInterfaceSm2DecodeKey(){
        if(!StringUtils.isBlank(this.interfaceSm2DecodeKey)){
            return this.interfaceSm2DecodeKey;
        }else{
            this.interfaceSm2DecodeKey = getField("InterfaceSm2DecodeKey");
            return this.interfaceSm2DecodeKey;
        }
    }
    public String getInterfaceSm2EncodeKey(){
        if(!StringUtils.isBlank(this.interfaceSm2EncodeKey)){
            return this.interfaceSm2EncodeKey;
        }else {
            this.interfaceSm2EncodeKey = getField("InterfaceSm2EncodeKey");
            return this.interfaceSm2EncodeKey;
        }
    }
    public List<String> getInterfaceEncryptList(){
        if(CollectionUtils.isNotEmpty(this.interfaceEncryptList)){
            return this.interfaceEncryptList;
        }else {
            List<String> list = new ArrayList<String>();
            list = getFieldsInOneFiled("IneterfaceEcnrypt", "patternUrl");
            this.interfaceEncryptList = list;
            return list;
        }
    }

    public void setConfigUrl(Resource configUrl) {
        this.configUrl = configUrl;
    }


    public List<String> getGlobalEnableFields(){
        if(globalEnableFields == null) {
            globalEnableFields = getArrFields("GlobalEnableFields");
        }
        return globalEnableFields;
    }

    public List<Map> getGlobalDisableMapFields()  {
        if(globalDisableMapFields == null){
            List<String> globalDisableEntityFieldList = getArrFields("GlobalDisableMapFields");
            if(CollectionUtils.isNotEmpty(globalDisableEntityFieldList)){
                globalDisableMapFields = new ArrayList<Map>();
                for (String s : globalDisableEntityFieldList) {
                    int i = s.lastIndexOf(".");
                    String mapid = s.substring(0, i);
                    String fieldName = s.substring(i+1);
                    Map entityDisableMap = new HashMap<String,Object>();
                    entityDisableMap.put("mapid",mapid);
                    entityDisableMap.put("field",fieldName);
                    globalDisableMapFields.add(entityDisableMap);
                }
            }
        }
        return globalDisableMapFields;
    }

    public List<Map> getGlobalDisableEntityFields()  {
        if(globalDisableEntityFields == null){
            List<String> globalDisableEntityFieldList = getArrFields("GlobalDisableEntityFields");
            if(CollectionUtils.isNotEmpty(globalDisableEntityFieldList)){
                globalDisableEntityFields = new ArrayList<Map>();
                for (String s : globalDisableEntityFieldList) {
                    int i = s.lastIndexOf(".");
                    String beanName = s.substring(0, i);
                    String fieldName = s.substring(i+1);
                    Map entityDisableEntityMap = new HashMap<String,Object>();
                    entityDisableEntityMap.put("entity",beanName);
                    entityDisableEntityMap.put("field",fieldName);
                    globalDisableEntityFields.add(entityDisableEntityMap);
                }
            }
        }
        return globalDisableEntityFields;
    }

    public List<Map> getSqlFileds()  {
        if(sqlFileds == null){
            List<String> getSqlFiledsList = getArrFields("SqlFileds");
            if(CollectionUtils.isNotEmpty(getSqlFiledsList)){
                sqlFileds = new ArrayList<Map>();
                for (String s : getSqlFiledsList) {
                    int i = s.lastIndexOf(".");
                    String mapid = s.substring(0, i);
                    String fieldName = s.substring(i+1);
                    Map sqlFiledsMap = new HashMap<String,Object>();
                    sqlFiledsMap.put("mapid",mapid);
                    sqlFiledsMap.put("field",fieldName);
                    sqlFileds.add(sqlFiledsMap);
                }
            }
        }
        return sqlFileds;
    }

    public List<Map> getSqlEncryptFileds()  {
        if(sqlEncryptFileds == null){
            List<String> getSqlEncryptFiledsList = getArrFields("SqlEncryptFileds");
            if(CollectionUtils.isNotEmpty(getSqlEncryptFiledsList)){
                sqlEncryptFileds = new ArrayList<Map>();
                for (String s : getSqlEncryptFiledsList) {
                    int i = s.lastIndexOf(".");
                    String mapid = s.substring(0, i);
                    String fieldName = s.substring(i+1);
                    Map sqlEncryptFiledsMap = new HashMap<String,Object>();
                    sqlEncryptFiledsMap.put("mapid",mapid);
                    sqlEncryptFiledsMap.put("field",fieldName);
                    sqlEncryptFileds.add(sqlEncryptFiledsMap);
                }
            }
        }
        return sqlEncryptFileds;
    }

    public List<Map> getSqlDecryptFileds()  {
        if(sqlDecryptFileds == null){
            List<String> getSqlDecryptFiledsList = getArrFields("SqlDecryptFileds");
            if(CollectionUtils.isNotEmpty(getSqlDecryptFiledsList)){
                sqlDecryptFileds = new ArrayList<Map>();
                for (String s : getSqlDecryptFiledsList) {
                    int i = s.lastIndexOf(".");
                    String mapid = s.substring(0, i);
                    String fieldName = s.substring(i+1);
                    Map sqlDecryptFiledsMap = new HashMap<String,Object>();
                    sqlDecryptFiledsMap.put("mapid",mapid);
                    sqlDecryptFiledsMap.put("field",fieldName);
                    sqlDecryptFileds.add(sqlDecryptFiledsMap);
                }
            }
        }
        return sqlDecryptFileds;
    }

    public List<Map> getEntityFileds()  {
        if(entityFileds == null){
            List<String> getEntityFiledsList = getArrFields("EntityFileds");
            if(CollectionUtils.isNotEmpty(getEntityFiledsList)){
                entityFileds = new ArrayList<Map>();
                for (String s : getEntityFiledsList) {
                    int i = s.lastIndexOf(".");
                    String mapid = s.substring(0, i);
                    String fieldName = s.substring(i+1);
                    Map entityFiledsMap = new HashMap<String,Object>();
                    entityFiledsMap.put("entity",mapid);
                    entityFiledsMap.put("field",fieldName);
                    entityFileds.add(entityFiledsMap);
                }
            }
        }
        return entityFileds;
    }





    public List<String> getArrFields(String fieldName){
        SAXReader sr = new SAXReader();
        Document document = null;
        List<String> result = new ArrayList<String>();
        try {
            document = sr.read(configUrl.getFile());
        } catch (Exception e) {
            LOGGER.error(null, e);
        }
        if(document != null){
            Element root = document.getRootElement();
            if(root != null) {
                Element globalEnableFields = root.element(fieldName);
                if(globalEnableFields != null){
                    //((DefaultText) ((DefaultElement) globalEnableFields).content).text.trim();
                    String value = globalEnableFields.elementTextTrim("value");
                    if(StringUtils.isNotBlank(value)){
                        String[] split = value.split(",");
                        if(split != null && split.length > 0){
                            result = Arrays.asList(split);
                        }
                    }
                }
            }
        }
        return result;
    }
    public List<String> getFieldsInOneFiled(String fieldName,String secondFieldName){
        SAXReader sr = new SAXReader();
        Document document = null;
        List<String> result = new ArrayList<String>();
        try {
            document = sr.read(configUrl.getFile());
        } catch (Exception e) {
            LOGGER.error(null, e);
        }
        if(document != null){
            Element root = document.getRootElement();
            if(root != null) {
                Element globalEnableFields = root.element(fieldName);
                if(globalEnableFields != null){
                    if(globalEnableFields.elements() != null && CollectionUtils.isNotEmpty(globalEnableFields.elements())){
                        List elements = globalEnableFields.elements();
                        for (Object obj : elements) {
                            Element element = (Element)obj;
                            List content = element.content();
                            if(CollectionUtils.isNotEmpty(content)){
                                DefaultText defaultText = (DefaultText)content.get(0);
                                if(defaultText != null && defaultText.getParent() != null
                                        && defaultText.getParent().getQName() != null
                                        && StringUtils.isNotBlank(defaultText.getParent().getQName().getName())
                                        && StringUtils.equals(secondFieldName,defaultText.getParent().getQName().getName()) ){
                                    String text = defaultText.getText();
                                    if(StringUtils.isNotBlank(text)){
                                        result.add(text);
                                    }
                                }
                            }
                        }
                    }
                }
            }
        }
        return result;
    }
    public String getField(String fieldName){
        SAXReader sr = new SAXReader();
        Document document = null;
        String value = "";
        try {
            document = sr.read(configUrl.getFile());
        } catch (Exception e) {
            LOGGER.error(null, e);
        }
        if(document != null){
            Element root = document.getRootElement();
            if(root != null) {
                Element globalEnableFields = root.element(fieldName);
                if(globalEnableFields != null){
                    //((DefaultText) ((DefaultElement) globalEnableFields).content).text.trim();
                    value = globalEnableFields.elementTextTrim("value");
                }
            }
        }
        return value;
    }
    public boolean getBooleanField(String fieldName){
        SAXReader sr = new SAXReader();
        Document document = null;
        try {
            document = sr.read(configUrl.getFile());
        } catch (Exception e) {
            LOGGER.error(null, e);
        }
        if(document != null){
            Element root = document.getRootElement();
            if(root != null) {
                Element globalEnableFields = root.element(fieldName);
                if(globalEnableFields != null){
                    //((DefaultText) ((DefaultElement) globalEnableFields).content).text.trim();
                    String stringVal = globalEnableFields.elementTextTrim("value");
                    if(StringUtils.isNotBlank(stringVal) && StringUtils.equals("true",stringVal)){
                        return true;
                    }
                }
            }
        }
        return false;
    }
}
